@@ -15,18 +15,13 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef _GST_JACK_H_
 #define _GST_JACK_H_
 
-#include <gst/audio/audio.h>
-#include "gstjackloader.h"
-
-GST_ELEMENT_REGISTER_DECLARE (jackaudiosrc);
-GST_ELEMENT_REGISTER_DECLARE (jackaudiosink);
 
 /**
  * GstJackConnect:
@@ -45,17 +40,7 @@ GST_ELEMENT_REGISTER_DECLARE (jackaudiosink);
 typedef enum {
   GST_JACK_CONNECT_NONE,
   GST_JACK_CONNECT_AUTO,
-  GST_JACK_CONNECT_AUTO_FORCED,
-
-  /**
-   * GstJackConnect::explicit
-   *
-   * In this mode, the element will try to connect to explicitly requested
-   * port specified by "port-names".
-   *
-   * Since: 1.20
-   */
-  GST_JACK_CONNECT_EXPLICIT,
+  GST_JACK_CONNECT_AUTO_FORCED
 } GstJackConnect;
 
 /**
