@@ -36,8 +36,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef __GST_JACK_AUDIO_SRC_H__
@@ -53,9 +53,15 @@
 
 G_BEGIN_DECLS
 
-#define GST_TYPE_JACK_AUDIO_SRC (gst_jack_audio_src_get_type())
-G_DECLARE_FINAL_TYPE (GstJackAudioSrc, gst_jack_audio_src,
-    GST, JACK_AUDIO_SRC, GstAudioBaseSrc)
+#define GST_TYPE_JACK_AUDIO_SRC             (gst_jack_audio_src_get_type())
+#define GST_JACK_AUDIO_SRC(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_JACK_AUDIO_SRC,GstJackAudioSrc))
+#define GST_JACK_AUDIO_SRC_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_JACK_AUDIO_SRC,GstJackAudioSrcClass))
+#define GST_JACK_AUDIO_SRC_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj),GST_TYPE_JACK_AUDIO_SRC,GstJackAudioSrcClass))
+#define GST_IS_JACK_AUDIO_SRC(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_JACK_AUDIO_SRC))
+#define GST_IS_JACK_AUDIO_SRC_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_JACK_AUDIO_SRC))
+
+typedef struct _GstJackAudioSrc GstJackAudioSrc;
+typedef struct _GstJackAudioSrcClass GstJackAudioSrcClass;
 
 struct _GstJackAudioSrc
 {
@@ -70,10 +76,7 @@ struct _GstJackAudioSrc
     gchar           *server;
     jack_client_t   *jclient;
     gchar           *client_name;
-    gchar           *port_pattern;
     guint            transport;
-    gboolean         low_latency;
-    gchar           *port_names;
 
     /* our client */
     GstJackAudioClient *client;
@@ -84,6 +87,13 @@ struct _GstJackAudioSrc
     sample_t **buffers;
 };
 
+struct _GstJackAudioSrcClass
+{
+    GstAudioBaseSrcClass parent_class;
+};
+
+GType gst_jack_audio_src_get_type (void);
+
 G_END_DECLS
 
 #endif /* __GST_JACK_AUDIO_SRC_H__ */
