@@ -13,30 +13,30 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 /**
  * SECTION:element-hdv1394src
- * @title: hdv1394src
  *
  * Read MPEG-TS data from firewire port.
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
  * gst-launch-1.0 hdv1394src ! queue ! decodebin name=d ! queue ! xvimagesink d. ! queue ! alsasink
  * ]| captures from the firewire port and plays the streams.
  * |[
  * gst-launch-1.0 hdv1394src ! queue ! filesink location=mydump.ts
  * ]| capture to a disk file
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 #include <unistd.h>
-#include <poll.h>
+#include <sys/poll.h>
 #include <sys/socket.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -119,8 +119,6 @@ static void gst_hdv1394src_update_device_name (GstHDV1394Src * src);
 G_DEFINE_TYPE_WITH_CODE (GstHDV1394Src, gst_hdv1394src, GST_TYPE_PUSH_SRC,
     G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER,
         gst_hdv1394src_uri_handler_init));
-GST_ELEMENT_REGISTER_DEFINE (hdv1394src, "hdv1394src", GST_RANK_NONE,
-    GST_TYPE_HDV1394SRC);
 
 static void
 gst_hdv1394src_class_init (GstHDV1394SrcClass * klass)
@@ -154,10 +152,12 @@ gst_hdv1394src_class_init (GstHDV1394SrcClass * klass)
           "like 0xhhhhhhhhhhhhhhhh. (0 = no guid)", 0, G_MAXUINT64,
           DEFAULT_GUID, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstHDV1394Src:device-name:
+   * GstHDV1394Src:device-name
    *
    * Descriptive name of the currently opened device
-   */
+   *
+   * Since: 0.10.7
+   **/
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "device name",
           "user-friendly name of the device", "Default",
@@ -170,7 +170,8 @@ gst_hdv1394src_class_init (GstHDV1394SrcClass * klass)
 
   gstpushsrc_class->create = gst_hdv1394src_create;
 
-  gst_element_class_add_static_pad_template (gstelement_class, &src_factory);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_factory));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "Firewire (1394) HDV video source", "Source/Video",
@@ -528,12 +529,8 @@ gst_hdv1394src_start (GstBaseSrc * bsrc)
   READ_SOCKET (src) = control_sock[0];
   WRITE_SOCKET (src) = control_sock[1];
 
-  if (fcntl (READ_SOCKET (src), F_SETFL, O_NONBLOCK) < 0)
-    GST_ERROR_OBJECT (src, "failed to make read socket non-blocking: %s",
-        g_strerror (errno));
-  if (fcntl (WRITE_SOCKET (src), F_SETFL, O_NONBLOCK) < 0)
-    GST_ERROR_OBJECT (src, "failed to make write socket non-blocking: %s",
-        g_strerror (errno));
+  fcntl (READ_SOCKET (src), F_SETFL, O_NONBLOCK);
+  fcntl (WRITE_SOCKET (src), F_SETFL, O_NONBLOCK);
 
   src->handle = raw1394_new_handle ();
 
@@ -565,22 +562,6 @@ gst_hdv1394src_start (GstBaseSrc * bsrc)
   raw1394_set_userdata (src->handle, src);
   raw1394_set_bus_reset_handler (src->handle, gst_hdv1394src_bus_reset);
 
-  {
-    nodeid_t m_node = (src->avc_node | 0xffc0);
-    int m_channel = -1;
-    int m_bandwidth = 0;
-    int m_outputPort = -1;
-    int m_inputPort = -1;
-
-    m_channel = iec61883_cmp_connect (src->handle, m_node, &m_outputPort,
-        raw1394_get_local_id (src->handle), &m_inputPort, &m_bandwidth);
-
-    if (m_channel >= 0) {
-      src->channel = m_channel;
-    }
-  }
-
-
   if ((src->iec61883mpeg2 =
           iec61883_mpeg2_recv_init (src->handle,
               gst_hdv1394src_iec61883_receive, src)) == NULL)
