@@ -15,25 +15,41 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "gstmatroskaelements.h"
+#include "matroska-demux.h"
+#include "matroska-parse.h"
+#include "matroska-read-common.h"
+#include "matroska-mux.h"
+#include "matroska-ids.h"
+#include "webm-mux.h"
+
+#include <gst/pbutils/pbutils.h>
 
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  gboolean ret = FALSE;
+  gboolean ret;
 
-  ret |= GST_ELEMENT_REGISTER (matroskademux, plugin);
-  ret |= GST_ELEMENT_REGISTER (matroskaparse, plugin);
-  ret |= GST_ELEMENT_REGISTER (matroskamux, plugin);
-  ret |= GST_ELEMENT_REGISTER (webmmux, plugin);
+  gst_pb_utils_init ();
+
+  gst_matroska_register_tags ();
+
+  GST_DEBUG_CATEGORY_INIT (matroskareadcommon_debug, "matroskareadcommon", 0,
+      "Matroska demuxer/parser shared debug");
+
+  ret = gst_matroska_demux_plugin_init (plugin);
+  ret &= gst_matroska_parse_plugin_init (plugin);
+  ret &= gst_element_register (plugin, "matroskamux", GST_RANK_PRIMARY,
+      GST_TYPE_MATROSKA_MUX);
+  ret &= gst_element_register (plugin, "webmmux", GST_RANK_PRIMARY,
+      GST_TYPE_WEBM_MUX);
 
   return ret;
 }
