@@ -13,47 +13,30 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:element-rtpamrdepay
- * @title: rtpamrdepay
- * @see_also: rtpamrpay
- *
- * Extract AMR audio from RTP packets according to RFC 3267.
- * For detailed information see: http://www.rfc-editor.org/rfc/rfc3267.txt
- *
- * ## Example pipeline
- * |[
- * gst-launch-1.0 udpsrc caps='application/x-rtp, media=(string)audio, clock-rate=(int)8000, encoding-name=(string)AMR, encoding-params=(string)1, octet-align=(string)1, payload=(int)96' ! rtpamrdepay ! amrnbdec ! pulsesink
- * ]| This example pipeline will depayload and decode an RTP AMR stream. Refer to
- * the rtpamrpay example to create the RTP stream.
- *
- */
-
-/*
- * RFC 3267 - Real-Time Transport Protocol (RTP) Payload Format and File
- * Storage Format for the Adaptive Multi-Rate (AMR) and Adaptive Multi-Rate
- * Wideband (AMR-WB) Audio Codecs.
- *
- */
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
 #include <gst/rtp/gstrtpbuffer.h>
-#include <gst/audio/audio.h>
 
 #include <stdlib.h>
 #include <string.h>
 #include "gstrtpamrdepay.h"
-#include "gstrtputils.h"
 
 GST_DEBUG_CATEGORY_STATIC (rtpamrdepay_debug);
 #define GST_CAT_DEFAULT (rtpamrdepay_debug)
 
+/* references:
+ *
+ * RFC 3267 - Real-Time Transport Protocol (RTP) Payload Format and File
+ * Storage Format for the Adaptive Multi-Rate (AMR) and Adaptive Multi-Rate
+ * Wideband (AMR-WB) Audio Codecs.
+ */
+
 /* RtpAMRDepay signals and args */
 enum
 {
@@ -63,7 +46,7 @@ enum
 
 enum
 {
-  PROP_0
+  ARG_0
 };
 
 /* input is an RTP packet
@@ -76,17 +59,17 @@ static GstStaticPadTemplate gst_rtp_amr_depay_sink_template =
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"audio\", "
-        "clock-rate = (int) 8000, " "encoding-name = (string) \"AMR\", "
-        /* This is the default, so the peer doesn't have to specify it
-         * "encoding-params = (string) \"1\", " */
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) 8000, "
+        "encoding-name = (string) \"AMR\", "
+        "encoding-params = (string) \"1\", "
         /* NOTE that all values must be strings in orde to be able to do SDP <->
          * GstCaps mapping. */
-        "octet-align = (string) \"1\";"
+        "octet-align = (string) \"1\", "
+        "crc = (string) { \"0\", \"1\" }, "
+        "robust-sorting = (string) \"0\", " "interleaving = (string) \"0\";"
         /* following options are not needed for a decoder
          *
-         "crc = (string) { \"0\", \"1\" }, "
-         "robust-sorting = (string) \"0\", "
-         "interleaving = (string) \"0\";"
          "mode-set = (int) [ 0, 7 ], "
          "mode-change-period = (int) [ 1, MAX ], "
          "mode-change-neighbor = (boolean) { TRUE, FALSE }, "
@@ -95,17 +78,17 @@ static GstStaticPadTemplate gst_rtp_amr_depay_sink_template =
          */
         "application/x-rtp, "
         "media = (string) \"audio\", "
-        "clock-rate = (int) 16000, " "encoding-name = (string) \"AMR-WB\", "
-        /* This is the default, so the peer doesn't have to specify it
-         * "encoding-params = (string) \"1\", " */
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) 16000, "
+        "encoding-name = (string) \"AMR-WB\", "
+        "encoding-params = (string) \"1\", "
         /* NOTE that all values must be strings in orde to be able to do SDP <->
          * GstCaps mapping. */
-        "octet-align = (string) \"1\";"
+        "octet-align = (string) \"1\", "
+        "crc = (string) { \"0\", \"1\" }, "
+        "robust-sorting = (string) \"0\", " "interleaving = (string) \"0\""
         /* following options are not needed for a decoder
          *
-         "crc = (string) { \"0\", \"1\" }, "
-         "robust-sorting = (string) \"0\", "
-         "interleaving = (string) \"0\""
          "mode-set = (int) [ 0, 7 ], "
          "mode-change-period = (int) [ 1, MAX ], "
          "mode-change-neighbor = (boolean) { TRUE, FALSE }, "
@@ -126,7 +109,7 @@ static GstStaticPadTemplate gst_rtp_amr_depay_src_template =
 static gboolean gst_rtp_amr_depay_setcaps (GstRTPBaseDepayload * depayload,
     GstCaps * caps);
 static GstBuffer *gst_rtp_amr_depay_process (GstRTPBaseDepayload * depayload,
-    GstRTPBuffer * rtp);
+    GstBuffer * buf);
 
 #define gst_rtp_amr_depay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpAMRDepay, gst_rtp_amr_depay, GST_TYPE_RTP_BASE_DEPAYLOAD);
@@ -140,17 +123,17 @@ gst_rtp_amr_depay_class_init (GstRtpAMRDepayClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstrtpbasedepayload_class = (GstRTPBaseDepayloadClass *) klass;
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_amr_depay_src_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_amr_depay_sink_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_amr_depay_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_amr_depay_sink_template));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP AMR depayloader", "Codec/Depayloader/Network/RTP",
       "Extracts AMR or AMR-WB audio from RTP packets (RFC 3267)",
       "Wim Taymans <wim.taymans@gmail.com>");
 
-  gstrtpbasedepayload_class->process_rtp_packet = gst_rtp_amr_depay_process;
+  gstrtpbasedepayload_class->process = gst_rtp_amr_depay_process;
   gstrtpbasedepayload_class->set_caps = gst_rtp_amr_depay_setcaps;
 
   GST_DEBUG_CATEGORY_INIT (rtpamrdepay_debug, "rtpamrdepay", 0,
@@ -283,12 +266,13 @@ static const gint wb_frame_size[16] = {
 };
 
 static GstBuffer *
-gst_rtp_amr_depay_process (GstRTPBaseDepayload * depayload, GstRTPBuffer * rtp)
+gst_rtp_amr_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
 {
   GstRtpAMRDepay *rtpamrdepay;
   const gint *frame_size;
   GstBuffer *outbuf = NULL;
   gint payload_len;
+  GstRTPBuffer rtp = { NULL };
   GstMapInfo map;
 
   rtpamrdepay = GST_RTP_AMR_DEPAY (depayload);
@@ -299,6 +283,8 @@ gst_rtp_amr_depay_process (GstRTPBaseDepayload * depayload, GstRTPBuffer * rtp)
   else
     frame_size = wb_frame_size;
 
+  gst_rtp_buffer_map (buf, GST_MAP_READ, &rtp);
+
   /* when we get here, 1 channel, 8000/16000 Hz, octet aligned, no CRC,
    * no robust sorting, no interleaving data is to be depayloaded */
   {
@@ -307,13 +293,13 @@ gst_rtp_amr_depay_process (GstRTPBaseDepayload * depayload, GstRTPBuffer * rtp)
     gint amr_len;
     gint ILL, ILP;
 
-    payload_len = gst_rtp_buffer_get_payload_len (rtp);
+    payload_len = gst_rtp_buffer_get_payload_len (&rtp);
 
     /* need at least 2 bytes for the header */
     if (payload_len < 2)
       goto too_small;
 
-    payload = gst_rtp_buffer_get_payload (rtp);
+    payload = gst_rtp_buffer_get_payload (&rtp);
 
     /* depay CMR. The CMR is used by the sender to request
      * a new encoding mode.
@@ -418,18 +404,17 @@ gst_rtp_amr_depay_process (GstRTPBaseDepayload * depayload, GstRTPBuffer * rtp)
     /* we can set the duration because each packet is 20 milliseconds */
     GST_BUFFER_DURATION (outbuf) = num_packets * 20 * GST_MSECOND;
 
-    if (gst_rtp_buffer_get_marker (rtp)) {
-      /* marker bit marks a buffer after a talkspurt. */
+    if (gst_rtp_buffer_get_marker (&rtp)) {
+      /* marker bit marks a discont buffer after a talkspurt. */
       GST_DEBUG_OBJECT (depayload, "marker bit was set");
-      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_RESYNC);
+      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
     }
 
     GST_DEBUG_OBJECT (depayload, "pushing buffer of size %" G_GSIZE_FORMAT,
         gst_buffer_get_size (outbuf));
-
-    gst_rtp_copy_audio_meta (rtpamrdepay, outbuf, rtp->buffer);
   }
 
+  gst_rtp_buffer_unmap (&rtp);
   return outbuf;
 
   /* ERRORS */
@@ -466,6 +451,7 @@ wrong_length_2:
 bad_packet:
   {
     /* no fatal error */
+    gst_rtp_buffer_unmap (&rtp);
     return NULL;
   }
 }
