@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef __GST_RTP_H264_DEPAY_H__
@@ -59,7 +59,6 @@ struct _GstRtpH264Depay
 
   /* Work around broken payloaders wrt. FU-A & FU-B */
   guint8 current_fu_type;
-  guint16 last_fu_seqnum;
   GstClockTime fu_timestamp;
   gboolean fu_marker;
 
@@ -67,14 +66,6 @@ struct _GstRtpH264Depay
   GPtrArray *sps;
   GPtrArray *pps;
   gboolean new_codec_data;
-
-  /* downstream allocator */
-  GstAllocator *allocator;
-  GstAllocationParams params;
-
-  gboolean wait_for_keyframe;
-  gboolean request_keyframe;
-  gboolean waiting_for_keyframe;
 };
 
 struct _GstRtpH264DepayClass
@@ -84,8 +75,7 @@ struct _GstRtpH264DepayClass
 
 GType gst_rtp_h264_depay_get_type (void);
 
-gboolean gst_rtp_h264_add_sps_pps (GstElement * rtph264, GPtrArray * sps,
-    GPtrArray * pps, GstBuffer * nal);
+gboolean gst_rtp_h264_depay_plugin_init (GstPlugin * plugin);
 
 G_END_DECLS
 
