@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef __GST_RTP_THEORA_PAY_H__
@@ -50,7 +50,6 @@ struct _GstRtpTheoraPay
 
   /* queues of buffers along with some stats. */
   GstBuffer    *packet;
-  GList        *packet_buffers;
   guint         payload_pos;
   guint         payload_left;
   guint32       payload_ident;
@@ -67,7 +66,6 @@ struct _GstRtpTheoraPay
   guint         config_interval;
   GstClockTime  last_config;
 
-  gint          pixel_format;
   gint          width;
   gint          height;
 };
@@ -79,6 +77,8 @@ struct _GstRtpTheoraPayClass
 
 GType gst_rtp_theora_pay_get_type (void);
 
+gboolean gst_rtp_theora_pay_plugin_init (GstPlugin * plugin);
+
 G_END_DECLS
 
 #endif /* __GST_RTP_THEORA_PAY_H__ */
