@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <gst/gst.h>
@@ -68,9 +68,8 @@ struct _GstVideoBox
 
   gint box_left, box_right, box_top, box_bottom;
 
-  gint src_x, src_y;
-  gint dest_x, dest_y;
-  gint crop_w, crop_h;
+  gint border_left, border_right, border_top, border_bottom;
+  gint crop_left, crop_right, crop_top, crop_bottom;
 
   gdouble alpha;
   gdouble border_alpha;
@@ -90,6 +89,4 @@ struct _GstVideoBoxClass
 
 GType gst_video_box_get_type (void);
 
-GST_ELEMENT_REGISTER_DECLARE (videobox);
-
 #endif /* __GST_VIDEO_BOX_H__ */
