@@ -791,7 +791,7 @@ gst_videomixer2_fill_queues (GstVideoMixer2 * mix,
       }
     } else {
       if (mixcol->end_time != -1) {
-        if (mixcol->end_time < output_start_time) {
+        if (mixcol->end_time <= output_start_time) {
           gst_buffer_replace (&mixcol->buffer, NULL);
           mixcol->start_time = mixcol->end_time = -1;
           if (!GST_COLLECT_PADS_STATE_IS_SET (mixcol,
@@ -1084,6 +1084,7 @@ gst_videomixer2_query_caps (GstPad * pad, GstObject * parent, GstQuery * query)
     }
   }
   gst_query_set_caps_result (query, caps);
+  gst_caps_unref (caps);
 
   return TRUE;
 }
