@@ -683,6 +683,10 @@ gst_osx_video_sink_set_window_handle (GstVideoOverlay * overlay, guintptr handle
         osxvideosink->osxwindow->gstview,
         @selector(addToSuperview:), osxvideosink->superview, YES);
   }
+
+  if (window_id) {
+    osxvideosink->osxwindow->internal = FALSE;
+  }
 }
 
 static void
@@ -837,7 +841,13 @@ gst_osx_video_sink_get_type (void)
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 
   GST_INFO_OBJECT (osxvideosink, "resizing");
-  [osxwindow->gstview setVideoSize:osxwindow->width :osxwindow->height];
+  NSSize size = {osxwindow->width, osxwindow->height};
+  if (osxwindow->internal) {
+    [osxwindow->win setContentSize:size];
+  }
+  if (osxwindow->gstview) {
+      [osxwindow->gstview setVideoSize :(int)osxwindow->width :(int)osxwindow->height];
+  }
   GST_INFO_OBJECT (osxvideosink, "done");
 
   [pool release];
