@@ -12,8 +12,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef __GST_XIMAGE_SRC_H__
@@ -49,15 +49,12 @@ struct _GstXImageSrc
 
   /* Information on display */
   GstXContext *xcontext;
-  gint x;
-  gint y;
   gint width;
   gint height;
 
   Window xwindow;
   gchar *display_name;
-  gint xwin_width;
-  gint xwin_height;
+  guint screen_num;
 
   /* Window selection */
   guint64 xid;
@@ -90,16 +87,9 @@ struct _GstXImageSrc
   guint endx;
   guint endy;
 
-  /* whether the user has put in specific coordinates for endx and/or endy */
-  gboolean endx_fit_to_screen;
-  gboolean endy_fit_to_screen;
-
   /* whether to use remote friendly calls */
   gboolean remote;
 
-  /* enable navigation events */
-  gboolean enable_navigation_events;
-
 #ifdef HAVE_XFIXES
   int fixes_event_base;
   XFixesCursorImage *cursor_image;
